import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_notification_bloc/src/domain/data_sources/remote/notification_remote_data_source.dart';
import 'package:wyatt_notification_bloc/src/domain/entities/remote_notifications.dart';
import 'package:wyatt_notification_bloc/src/domain/repositories/notification_repository.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

class NotificationRepositoryImpl extends NotificationRepository {
  final NotificationRemoteDataSource _notificationRemoteDataSource;

  NotificationRepositoryImpl({
    required NotificationRemoteDataSource notificationRemoteDataSource,
  }) : _notificationRemoteDataSource = notificationRemoteDataSource;
  @override
  FutureOrResult<void> register() => Result.tryCatchAsync(
        _notificationRemoteDataSource.register,
        (error) => ClientException(error.toString()),
      );

  @override
  FutureOrResult<String> getToken() => Result.tryCatchAsync(
        _notificationRemoteDataSource.getToken,
        (error) => ClientException(error.toString()),
      );

  @override
  FutureOrResult<Stream<RemoteNotificationWrapper>> onNotification() =>
      Result.tryCatch(
        _notificationRemoteDataSource.onNotification,
        (error) => ClientException(error.toString()),
      );

  @override
  FutureOrResult<RemoteNotificationWrapper?> getInitialMessage() =>
      Result.tryCatchAsync(
        _notificationRemoteDataSource.getInitialMessage,
        (error) => ClientException(error.toString()),
      );

  @override
  FutureOrResult<Stream<RemoteNotificationWrapper>> onNotificationOpened() =>
      Result.tryCatch(
        _notificationRemoteDataSource.onNotificationOpened,
        (error) => ClientException(error.toString()),
      );

  @override
  FutureOrResult<void> subscribeToTopic(String topic) => Result.tryCatchAsync(
        () => _notificationRemoteDataSource.subscribeToTopic(topic),
        (error) => ClientException(error.toString()),
      );

  @override
  FutureOrResult<void> unsubscribeFromTopic(String topic) =>
      Result.tryCatchAsync(
        () => _notificationRemoteDataSource.unsubscribeFromTopic(topic),
        (error) => ClientException(error.toString()),
      );
}
