import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_notification_bloc/src/core/enum/notification_status.dart';
import 'package:wyatt_notification_bloc/src/domain/entities/remote_notifications.dart';
import 'package:wyatt_notification_bloc/src/domain/usecases/get_intitial_message.dart';
import 'package:wyatt_notification_bloc/src/domain/usecases/listen_notification.dart';
import 'package:wyatt_notification_bloc/src/domain/usecases/register.dart';
import 'package:wyatt_notification_bloc/src/domain/usecases/subscribe.dart';
import 'package:wyatt_notification_bloc/src/domain/usecases/unsubscribe.dart';

part 'notification_state.dart';

typedef OnNotification = void Function(RemoteNotificationWrapper);

class NotificationCubit extends Cubit<NotificationState> {
  final Register _register;
  final ListenNotification _listenNotification;
  final Subscribe _subscribe;
  final Unsubscribe _unsubscribe;
  final GetInitialMessage _getInitialMessage;

  final OnNotification? handleNotification;
  NotificationCubit({
    required Register register,
    required GetInitialMessage getInitialMessage,
    required ListenNotification listenNotification,
    required Subscribe subscribe,
    required Unsubscribe unsubscribe,
    this.handleNotification,
  })  : _register = register,
        _getInitialMessage = getInitialMessage,
        _listenNotification = listenNotification,
        _subscribe = subscribe,
        _unsubscribe = unsubscribe,
        super(const NotificationState.unknown());

  FutureOr<void> register() async {
    print("Registering Threw cubit\n");
    final response = await _register.execute(const NoParam());

    print(response);
    if (response.isOk) {
      emit(const NotificationState.authorized());

      final initialMessage = await _getInitialMessage.execute(const NoParam());
      if (initialMessage.isOk && initialMessage.ok != null) {
        handleNotification?.call(initialMessage.ok!);
      }
    } else if (response.isErr) {
      emit(const NotificationState.unauthorized());
    }
  }

  Future<void> listenNotification() async {
    print("Listening for notifications\n");
    final notificaitons = await _listenNotification.execute(const NoParam());
    if (notificaitons.isOk) {
      print("Get notifications ready to listen");
      notificaitons.ok?.listen((_) {
        print("New notif !!\n");
      });
    }
  }

  FutureOr<void> subscribe(String topic) => _subscribe.execute(topic);

  FutureOr<void> unsubscribe(String topic) => _unsubscribe.execute(topic);
}
